SystemOrganization addCategory: #'Sight-Tagging'!
SystemOrganization addCategory: #'Sight-Tagging-Tests'!

TestCase subclass: #SdTagTests
	instanceVariableNames: 'organizer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Tagging-Tests'!

!SdTagTests methodsFor: 'accessing' stamp: 'mt 7/27/2012 13:57:40.674'!
method1

	^ SdTag >> #tagname! !

!SdTagTests methodsFor: 'accessing' stamp: 'mt 7/27/2012 13:57:48.314'!
method2

	^ SdTag >> #description! !

!SdTagTests methodsFor: 'running' stamp: 'mt 7/27/2012 13:52:30.09'!
setUp

	super setUp.
	organizer := SdTagOrganizer new.! !

!SdTagTests methodsFor: 'running' stamp: 'mt 7/27/2012 16:22:57.852'!
tearDown

	self method1 resetMethodTags.
	self method2 resetMethodTags.

	SystemChangeNotifier uniqueInstance noMoreNotificationsFor: organizer.

	super tearDown.! !

!SdTagTests methodsFor: 'tests' stamp: 'mt 7/27/2012 14:11:46.903'!
test01AttachDetachTag

	| tag |
	tag := SdTag new tagname: 'testing'.
	
	self assert: self method1 methodTags isEmpty.
	organizer attachTag: tag toObject: self method1.
	self assert: self method1 methodTags notEmpty.
	organizer detachTag: tag fromObject: self method1.
	self assert: self method1 methodTags isEmpty.! !

!SdTagTests methodsFor: 'tests' stamp: 'mt 7/30/2012 15:30:02.239'!
test02OrganizerQuery

	| tag1 tag2 |
	tag1 := SdTag new tagname: 'testing1'.
	tag2 := SdTag new tagname: 'testing2'.
	
	organizer
		registerTag: tag1;
		registerTag: tag2;    
		attachTag: tag1 toObject: self method1;
		attachTag: tag1 toObject: self method2;
		attachTag: tag2 toObject: self method2.

	self
		assert: organizer tags asSet = {tag1. tag2} asSet;
		assert: (organizer tagNamed: 'testing1') == tag1;
		assert: (organizer objectsAt: tag1) asSet = {self method1. self method2} asSet;
		assert: (organizer objectsAtName: 'testing2') asSet = {self method2} asSet.! !

!SdTagTests methodsFor: 'tests' stamp: 'mt 7/27/2012 14:04:03.405'!
test03DestroyTag

	| tag |
	tag := SdTag new tagname: 'testing'.
	
	organizer
		attachTag: tag toObject: self method1;
		attachTag: tag toObject: self method2.

	self
		assert: self method1 hasMethodTags;
		assert: self method2 hasMethodTags.
		
	organizer destroyTag: tag.
	
	self
		deny: self method1 hasMethodTags;
		deny: self method2 hasMethodTags.
! !

!SdTagTests methodsFor: 'tests' stamp: 'mt 7/27/2012 16:22:37.539'!
test04SystemChangesMethodChanged

	| tag method |
	tag := SdTag new tagname: 'testing'.
	method := self method1.
	
	organizer listenToSystemChanges.
	organizer attachTag: tag toObject: self method1.
	
	"Simulate modification."
	self method1 methodClass
		compile: self method1 getSource
		classified: (self method1 methodClass organization categoryOfElement: self method1 selector).
		
	self
		assert: method ~~ self method1;
		assert: (organizer objectsAt: tag) asSet = {self method1} asSet;
		assert: self method1 methodTags notEmpty.! !

!SdTagTests methodsFor: 'tests' stamp: 'mt 7/27/2012 16:22:43.387'!
test05SystemChangesMethodRemoved

	| tag |
	tag := SdTag new tagname: 'testing'.
	
	organizer listenToSystemChanges.
	organizer attachTag: tag toObject: self method1.
	
	"Simulate modification."
	SystemChangeNotifier uniqueInstance
		methodRemoved: self method1
		selector: self method1 selector
		class: self method1 methodClass.  
		
	self assert: (organizer objectsAt: tag) isEmpty..! !

!SdTagTests methodsFor: 'tests' stamp: 'lt 8/1/2012 17:03:52.703'!
test06TagsForCategory

	{'Sight-Tagging-Tests' -> #(sight tagging tests).
	'*sight-tagging' -> #(extension sight tagging) } do: [:assoc |
		self
			assert: assoc value
			equals: ((organizer class tagsForCategory: assoc key) collect: [:tag | tag tagname])].! !

!CompiledMethod methodsFor: '*sight-tagging' stamp: 'mt 7/27/2012 11:38:15.882'!
attachTag: aTag
	
	self hasMethodTags 
	 	ifFalse: [ self methodTags: Set new ].
	self methodTags add: aTag.! !

!CompiledMethod methodsFor: '*sight-tagging' stamp: 'mt 7/27/2012 11:38:29.012'!
detachTag: aSdTag

	self methodTags remove: aSdTag.
	self methodTags ifEmpty: [ self resetMethodTags ].! !

!CompiledMethod methodsFor: '*sight-tagging' stamp: 'mt 7/27/2012 10:28:23.754'!
hasMethodTags

	^ self properties includesKey: #methodTags! !

!CompiledMethod methodsFor: '*sight-tagging' stamp: 'mt 7/27/2012 10:28:35.781'!
methodTags

	^ self properties 
		at: #methodTags 
		ifAbsent: #()! !

!CompiledMethod methodsFor: '*sight-tagging' stamp: 'mt 7/27/2012 10:28:50.537'!
methodTags: aCollection

	aCollection ifNil: [^ self resetMethodTags ].
	
	^ self properties
		at: #methodTags
		put: aCollection ! !

!CompiledMethod methodsFor: '*sight-tagging' stamp: 'mt 7/27/2012 10:29:02.413'!
resetMethodTags

	self properties removeKey: #methodTags ifAbsent: [].
! !

Object subclass: #SdTag
	instanceVariableNames: 'tagname icon type description'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Tagging'!

!SdTag commentStamp: 'mt 7/27/2012 11:52:51.006' prior: 0!
A way to categorize objects in the system.!

!SdTag class methodsFor: 'instance creation' stamp: 'mt 7/30/2012 15:26:59.481'!
named: aName

	^ SdTagOrganizer default tags
		detect: [:tag | tag tagname = aName]
		ifNone: [SdTag new tagname: aName; register]! !

!SdTag methodsFor: 'organizing - attach/detach' stamp: 'mt 7/27/2012 11:49:40.392'!
attachTo: anObject

	SdTagOrganizer default attachTag: self toObject: anObject.! !

!SdTag methodsFor: 'accessing' stamp: 'sk 6/28/2012 09:26:35.466'!
description
	^ description! !

!SdTag methodsFor: 'accessing' stamp: 'sk 6/28/2012 09:26:36.249'!
description: aString
	description := aString
! !

!SdTag methodsFor: 'organizing - attach/detach' stamp: 'mt 7/27/2012 11:49:55.514'!
detachFrom: anObject

	SdTagOrganizer default detachTag: self fromObject: anObject.! !

!SdTag methodsFor: 'accessing' stamp: 'mt 7/30/2012 16:58:00.481'!
icon
	^ icon! !

!SdTag methodsFor: 'accessing' stamp: 'sk 6/20/2012 18:17:19.755'!
icon: anIcon
	icon := anIcon
! !

!SdTag methodsFor: 'printing' stamp: 'sk 6/19/2012 17:20:28.738'!
printOn: aStream
	aStream
		nextPutAll: 'SdTag[';
		nextPutAll: self tagname;
		nextPutAll: ']'.! !

!SdTag methodsFor: 'printing' stamp: 'sk 6/19/2012 17:21:34.893'!
printString

	^ 'SdTag[', self tagname, ']'! !

!SdTag methodsFor: 'organizing - registration' stamp: 'mt 7/30/2012 15:25:50.879'!
register

	SdTagOrganizer default registerTag: self.  ! !

!SdTag methodsFor: 'accessing' stamp: 'mt 7/30/2012 15:30:18.524'!
tagname
	^ tagname! !

!SdTag methodsFor: 'accessing' stamp: 'sk 6/19/2012 16:26:03.153'!
tagname: aString
	tagname := aString! !

!SdTag methodsFor: 'accessing' stamp: 'sk 6/22/2012 11:02:48.442'!
type
	^ type! !

!SdTag methodsFor: 'accessing' stamp: 'sk 6/22/2012 11:02:48.488'!
type: aSymbol
	type := aSymbol
! !

!SdTag methodsFor: 'organizing - registration' stamp: 'mt 7/30/2012 15:26:12.916'!
unregister

	SdTagOrganizer default unregisterTag: self.  ! !

Object subclass: #SdTagOrganizer
	instanceVariableNames: 'tags index'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sight-Tagging'!

SdTagOrganizer class
	instanceVariableNames: 'default'!

!SdTagOrganizer commentStamp: 'mt 7/27/2012 11:52:31.024' prior: 0!
Organizes tags in the system. Manages synonyms, groupings, ordering, etc.!

SdTagOrganizer class
	instanceVariableNames: 'default'!

!SdTagOrganizer class methodsFor: 'autotag' stamp: 'mt 7/30/2012 16:08:40.436'!
collectTagsForSystem

	| tagOrganizer |
	tagOrganizer := SdTagOrganizer default.
	
	SystemOrganization categories do: [:category |
		| classTags |
		classTags := SdTagOrganizer tagsForCategory: category.
		((SystemOrganization listAtCategoryNamed: category)
			gather: [:classname | {(Smalltalk at: classname). (Smalltalk at: classname) class}])
			do: [:cls |
				cls organization categories do: [:protocol |
					| protocolTags |
					protocolTags := SdTagOrganizer tagsForCategory: protocol.
					(cls organization listAtCategoryNamed: protocol) do: [:selector |
						classTags do: [:tag | tag attachTo: cls >> selector].
						protocolTags do: [:tag | tag attachTo: cls >> selector].
						]]]] displayingProgress: [:category | category].! !

!SdTagOrganizer class methodsFor: 'instance' stamp: 'mt 7/27/2012 16:33:37.072'!
default

	^ default ifNil: [default := SdTagOrganizer new listenToSystemChanges]! !

!SdTagOrganizer class methodsFor: 'instance' stamp: 'mt 7/30/2012 16:02:05.426'!
reset

	self default sanitize.
	default := nil.! !

!SdTagOrganizer class methodsFor: 'autotag' stamp: 'lt 8/1/2012 17:04:14.711'!
tagsForCategory: aString

	| extensionTags identifier |
	(aString beginsWith: '*')
		ifTrue: [identifier := aString allButFirst. extensionTags := {SdTag named: 'extension'}]
		ifFalse: [identifier := aString. extensionTags := {}].

	^ extensionTags, (((identifier subStrings: {$-.$/}) collect: [:token | token withBlanksTrimmed asLowercase])
		select: [:token | token notEmpty]
		thenCollect: [:token | SdTag named: token])! !

!SdTagOrganizer methodsFor: 'attach/detach' stamp: 'mt 7/27/2012 16:17:48.179'!
attachTag: aTag toObject: anObject

	self flag: #risky. "mt: Better use weak references in an identity set?"
	(index at: aTag ifAbsentPut: [IdentitySet new])
		add: anObject.
		
	anObject attachTag: aTag.  ! !

!SdTagOrganizer methodsFor: 'attach/detach' stamp: 'mt 7/30/2012 16:05:30.085'!
destroyTag: aTag

	(self objectsAt: aTag) do: [:object | object detachTag: aTag].
	index removeKey: aTag ifAbsent: [].! !

!SdTagOrganizer methodsFor: 'attach/detach' stamp: 'mt 7/27/2012 12:03:00.283'!
detachTag: aTag fromObject: anObject

	(index at: aTag ifAbsent: [^ self])
		remove: anObject ifAbsent: [].
		
	anObject detachTag: aTag.! !

!SdTagOrganizer methodsFor: 'initialization' stamp: 'mt 7/30/2012 15:23:19.633'!
initialize

	super initialize.
	
	tags := Dictionary new.
	index := IdentityDictionary new.! !

!SdTagOrganizer methodsFor: 'system changes' stamp: 'mt 7/27/2012 16:22:09.47'!
listenToSystemChanges

	SystemChangeNotifier uniqueInstance notify: self ofSystemChangesOfItem: AbstractEvent methodKind change: ModifiedEvent changeKind using: #onMethodModified:.
	SystemChangeNotifier uniqueInstance notify: self ofSystemChangesOfItem: AbstractEvent methodKind change: RemovedEvent  changeKind using: #onMethodRemoved:.! !

!SdTagOrganizer methodsFor: 'query' stamp: 'mt 7/27/2012 11:25:48.411'!
objectsAt: aTag

	^ (index at: aTag ifAbsent: [#()]) asOrderedCollection! !

!SdTagOrganizer methodsFor: 'query' stamp: 'mt 7/27/2012 11:29:11.454'!
objectsAtName: aString

	^ self objectsAt: (self tagNamed: aString)! !

!SdTagOrganizer methodsFor: 'system changes' stamp: 'mt 7/27/2012 16:21:33.672'!
onMethodModified: aSystemEvent

	aSystemEvent oldItem hasMethodTags ifTrue: [
		aSystemEvent oldItem methodTags copy do: [:tag |
			self
				attachTag: tag toObject: aSystemEvent item;
				detachTag: tag fromObject: aSystemEvent oldItem]].! !

!SdTagOrganizer methodsFor: 'system changes' stamp: 'mt 7/27/2012 12:01:07.736'!
onMethodRemoved: aSystemEvent

	aSystemEvent item hasMethodTags ifTrue: [
		aSystemEvent item methodTags copy do: [:tag |
			self detachTag: tag fromObject: aSystemEvent item]].! !

!SdTagOrganizer methodsFor: 'registration' stamp: 'mt 7/30/2012 15:24:36.634'!
registerTag: aTag

	tags at: aTag tagname put: aTag.! !

!SdTagOrganizer methodsFor: 'initialization' stamp: 'mt 7/27/2012 16:33:17.035'!
sanitize

	self tags do: [:tag | self destroyTag: tag].! !

!SdTagOrganizer methodsFor: 'query' stamp: 'mt 7/30/2012 15:28:56.188'!
tagNamed: aString

	^ self tagNamed: aString ifAbsent: []! !

!SdTagOrganizer methodsFor: 'query' stamp: 'mt 7/30/2012 15:28:45.553'!
tagNamed: aString ifAbsent: errorBlock

	^ tags at: aString ifAbsent: errorBlock! !

!SdTagOrganizer methodsFor: 'query' stamp: 'mt 7/30/2012 15:27:50.811'!
tags

	^ tags values! !

!SdTagOrganizer methodsFor: 'registration' stamp: 'mt 7/30/2012 15:25:07.508'!
unregisterTag: aTag

	tags removeKey: aTag tagname ifAbsent: [].
	self destroyTag: aTag.  ! !
